/*
 * EL4J, the Extension Library for the J2EE, adds incremental enhancements to
 * the spring framework, http://el4j.sf.net
 * Copyright (C) 2006 by ELCA Informatique SA, Av. de la Harpe 22-24,
 * 1000 Lausanne, Switzerland, http://www.elca.ch
 *
 * EL4J is published under the GNU Lesser General Public License (LGPL)
 * Version 2.1. See http://www.gnu.org/licenses/
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * For alternative licensing, please contact info@elca.ch
 */
package ch.elca.el4j.services.xmlmerge.factory;

import ch.elca.el4j.services.xmlmerge.AbstractXmlMergeException;
import ch.elca.el4j.services.xmlmerge.MatchException;
import ch.elca.el4j.services.xmlmerge.Operation;
import ch.elca.el4j.services.xmlmerge.OperationFactory;
import org.jaxen.JaxenException;
import org.jaxen.jdom.JDOMXPath;
import org.jdom.Element;
import org.jdom.Namespace;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

/**
 * An operation factory that resolves operations given a map { xpath (as
 * String), Operation }. The order in the map is relevant if several XPath
 * matches.
 *
 * @svnLink $Revision: 3884 $;$Date: 2009-08-04 15:48:31 +0200 (mar., 04 août 2009) $;$Author: swismer $;$URL: https://el4j.svn.sourceforge.net/svnroot/el4j/trunk/el4j/framework/modules/xml_merge/common/src/main/java/ch/elca/el4j/services/xmlmerge/factory/XPathOperationFactory.java $
 *
 * @author Laurent Bovet (LBO)
 * @author Alex Mathey (AMA)
 */
// Namespaces: Inheritance from BaseFactory to support namespaces
public class XPathOperationFactory extends BaseFactory implements OperationFactory {

	/**
	 * A map containing configuration properties.
	 */
	Map m_map = new HashMap();

	/**
	 * The default operation returned by this factory.
	 */
	Operation m_defaultOperation;

	/**
	 * Sets the factory's map containing configuration properties.
	 *
	 * @param map
	 *            A map containing configuration properties.
	 */
	public void setOperationMap(Map map) {
		this.m_map = map;
	}

	/**
	 * Sets the default operation returned by this factory.
	 * @param operation The default operation returned by this factory.
	 */
	public void setDefaultOperation(Operation operation) {
		this.m_defaultOperation = operation;
	}

	/**
	 * {@inheritDoc}
	 */
	public Operation getOperation(Element originalElement, Element patchElement)
		throws AbstractXmlMergeException {
		Iterator it = m_map.keySet().iterator();
		while (it.hasNext()) {
			String xPath = (String) it.next();
			if (matches(originalElement, xPath) || matches(patchElement,
				xPath)) {
				return (Operation) m_map.get(xPath);
			}
		}
		return m_defaultOperation;
	}

	/**
	 * Detects whether the given element matches the given XPath string.
	 *
	 * @param element
	 *            The element which will be checked
	 * @param xPathString
	 *            The XPath expression the element will be checked against
	 * @return True if the given element matches the given XPath string
	 * @throws AbstractXmlMergeException
	 *             If an error occurred during the matching process
	 */
	private boolean matches(Element element, String xPathString)
		throws AbstractXmlMergeException {

		if (element == null) {
			return false;
		}

		try {
			JDOMXPath xPath = new JDOMXPath(xPathString);

            // Namespaces: Add namespaces to xPath
            if (element.getNamespace() != null)
                xPath.addNamespace(element.getNamespace().getPrefix(), element.getNamespace().getURI());

            for(Namespace n: namespaces.values()) {
                xPath.addNamespace(n.getPrefix(),  n.getURI());
            }
            // End Namespaces: Add namespaces to xPath

			boolean result = xPath.selectNodes(element.getParent()).contains(
				element);

			return result;

		} catch (JaxenException e) {
			throw new MatchException(element, e);
		}
	}

}
