function correlated = correlate_position(fixed, to_adjust)
% CORRELATE_POSITION Correlation of channel positions.
%   CORRELATE_POSITION(FIXED, TO_ADJUST) calculates the shift to apply so
%   that the pixels of the spatial imageScan channel TO_ADJUST correlated
%   well with the pixels of the spatial imageScan channel FIXED and stores
%   any drift/real coordinate information that can be derived from this
%   shift in the channel parameters.

% Check input
try
    parser = inputParser;
    parser.addRequired('fixed', @(in)is_channel(in) && ...
        strcmp(fixed.type,'imageScan') && ...
        strcmp(fixed.pixel_domain,'spatial'));
    parser.addRequired('to_adjust', @(in)is_channel(in) && ...
        strcmp(to_adjust.type,'imageScan') && ...
        strcmp(to_adjust.pixel_domain,'spatial'));
    parser.parse(fixed, to_adjust);
catch error
    me = MException('spm:correlate_position:input_error', 'Invalid input: %s', ...
        error.message);
    throw(me);
end

if sum(size(fixed.pixels) ~= size(to_adjust.pixels)) ~= 0
    me = MException('spm:correlate_position:inconsistent_sizes', ...
        'Channel pixel matrices have different dimensions.');
    throw(me);
end

% Copy to_adjust to output
correlated = to_adjust;

% Perform a fast correlation
cor = fft2(fixed.pixels);
cor = abs(ifft2(conj(cor) .* fft2(to_adjust.pixels)));
cor=cor/length(fixed.pixels(:));%following PV-WAVE normalization

% Find the maximum in the cross correlation function
[~, imax] = max(cor(:));
[ypeak, xpeak] = ind2sub(size(cor),imax(1));
shift = [-(ypeak-1), -(xpeak-1)];

% Now translate to real-world coordinate system shift
if  to_adjust.axis(2).count > 0 % Otherwise shift should be zero
    y_shift = shift(1) * abs(to_adjust.axis(2).variable.extent) / ...
        to_adjust.axis(2).count;
end

if  to_adjust.axis(1).count > 0 % Otherwise shift should be zero
    x_shift =  shift(2) * abs( to_adjust.axis(1).variable.extent) / ...
        to_adjust.axis(1).count;
end

ix_to_adjust = to_adjust.axis(1).variable.ref_index;
iy_to_adjust = to_adjust.axis(2).variable.ref_index;
ix_fixed = fixed.axis(1).variable.ref_index;
iy_fixed = fixed.axis(2).variable.ref_index;

% Set shift in correlated output
correlated.parameter(ix_to_adjust).shift = x_shift;
correlated.parameter(iy_to_adjust).shift = y_shift;

% Adapt real coordinates and drift, if possible
if isnan(to_adjust.parameter(ix_to_adjust).start_value.real) && ...
        ~isnan(fixed.parameter(ix_fixed).start_value.real)
    correlated.parameter(ix_to_adjust).start_value.real = ...
        fixed.parameter(ix_fixed).start_value.real + x_shift;
    correlated.parameter(ix_to_adjust).start_value.drift = ...
        correlated.parameter(ix_to_adjust).start_value.real - ...
        correlated.parameter(ix_to_adjust).start_value.coarse - ...
        correlated.parameter(ix_to_adjust).start_value.fine;
end

if isnan(to_adjust.parameter(iy_to_adjust).start_value.real) && ...
        ~isnan(fixed.parameter(iy_fixed).start_value.real)
    correlated.parameter(iy_to_adjust).start_value.real = ...
        fixed.parameter(iy_fixed).start_value.real + y_shift;
    correlated.parameter(iy_to_adjust).start_value.drift = ...
        correlated.parameter(iy_to_adjust).start_value.real - ...
        correlated.parameter(iy_to_adjust).start_value.coarse - ...
        correlated.parameter(iy_to_adjust).start_value.fine;
end

% Save fixed channel to file
filename = tempname;
spm_write(fixed, filename);

% Add fixed channel to the context
correlated.context{length(correlated.context) + 1} = filename;

% Set the correct reference index
correlated.ref_index = length(correlated.context);

end

