function out = cut(in, varargin)
%CUT crop a channel to a region of interest.
%   CUT(IN, LOWER_X, LOWER_Y, UPPER_X, UPPER_Y) returns the region of
%   interest of the spatial imageScan channel IN delimited by the lower-
%   left corner (LOWER_X,LOWER_Y) and the right-upper corner 
%   (UPPER_X,UPPER_Y). 
%
%   The coordinate system, in which the corner coordinates are given is
%   indicated by a parameter 'axis_type', which equals one of 'none',
%   'relative', 'center', 'fine', 'coarse', 'real_fine', 'real_coarse'
%   (c.f. spm_display). By default axis_type is 'none', which implies that
%   one actually indicates the rows/columns of the pixel matrix to be
%   selected. In any of the other options the coordinates are expressed in
%   display units. When cutting out a region using the coordinates and 
%   units as displayed by spm_display, please ensure that in the display 
%   options both axes are set to the same axis type. This single axis type  
%   should then also be provided when calling CUT. When using the default 
%   parameter settings of spm_display, please select 'relative', as this 
%   is the default axis type for spm_display.
%
%   To ensure that no subsampling takes place, the function can be forced 
%   to snap to the grid by setting the parameter 'snap_to_grid' to true 
%   (by default it is false). So, for instance  the call CUT(IN, LOWER_X,
%   LOWER_Y, UPPER_X, UPPER_Y, 'axis_type', 'fine', 'snap_to_grid', true)
%   interprets the given corner coordinates in the fine coordinate system,
%   but snaps them to the grid before cutting. 
%
%   If the input channel is already displayed in a figure, it is also 
%   possible to call CUT in an interactive mode. Instead of providing 
%   a channel and corner coordinates, the user specifies a figure
%   handle/name on the command line: CUT(FIGID). The corners of the 
%   rectangle to be selected are then indicated by clicking in the figure 
%   window.

% Check input
try
    parser = inputParser;
    parser.addRequired('in', @(in)(is_channel(in)  && ...
        strcmp(in.type,'imageScan') && ...
        strcmp(in.pixel_domain,'spatial')));
    parser.addRequired('lower_x', @(lower_x)(isnumeric(lower_x) && ...
        numel(lower_x) == 1));
    parser.addRequired('lower_y', @(lower_y)(isnumeric(lower_y) && ...
        numel(lower_y) == 1));
    parser.addRequired('upper_x', @(upper_x)(isnumeric(upper_x) && ...
        numel(upper_x) == 1));
    parser.addRequired('upper_y', @(upper_y)(isnumeric(upper_y) && ...
        numel(upper_y) == 1));
    parser.addParamValue('axis_type', 'none', @(axis_type)(...
        ischar(axis_type) && ismember(axis_type, {'relative', 'center', ...
        'fine', 'coarse', 'real_fine', 'real_coarse', 'relative_real', ...
        'none'})));
    parser.addParamValue('snap_to_grid', false, @islogical);
    parser.parse(in, varargin{:});
    lower_x = parser.Results.lower_x;
    lower_y = parser.Results.lower_y;
    upper_x  =  parser.Results.upper_x;
    upper_y = parser.Results.upper_y;
    axis_type = parser.Results.axis_type;
    snap_to_grid = parser.Results.snap_to_grid;
catch
    try
        parser = inputParser;
        parser.addRequired('figid', @(figid) (length(figid) == 1 && ...
            (ishandle(figid) && ...
            ~isempty(findobj(figid,'type','figure'))) || ...
            (ischar(figid) && ...
            ~isempty(findobj('name', figid,'type','figure')))));
        parser.parse(in);
        figid = parser.Results.figid;
    catch error
        me = MException('spm:cut:input_error', ...
            'Invalid input: %s', error.message);
        throw(me);
    end
end

% Interactive version
if nargin == 1
    % Find figure
    if ishandle(figid)
        fig = figid;
    else
        fig = findobj('name', figid,'type','figure');
        if numel(fig) > 1
            fig = fig(1);
            warning('spm:cut:ambiguous_identifier', ...
                'More than one window with indicated name');
        end
    end

    % Try to fetch channel data
    in = get(fig,'UserData');
    if isempty(in)
        me = MException('spm:cut:no_channel_found', ...
            'Figure does not have a channel attached to it.');
        throw(me);
    end
    
    figure(fig);
    [x, y] = ginput(2);
    lower_x = min(x);
    upper_x = max(x);
    lower_y = min(y);
    upper_y = max(y);
    
    axis_type = 'none';
    snap_to_grid = false;
end

if lower_x > upper_x || lower_y > upper_y
    me = MException('spm:cut:empty_rectangle', ...
        'Selected rectangle is empty.');
    throw(me);
end

% Copy input to output
out = in;

% Check whether boundaries are allowed
[hor_min, hor_max, vert_min, vert_max] = get_axes_limits(in, ...
    axis_type, axis_type);
if strcmp(axis_type, 'none')
    hor_min = 1;
    vert_min = 1;
    hor_max = size(in.pixels,2);
    vert_max = size(in.pixels,1);
end
if upper_x > hor_max || lower_x < hor_min || upper_y > vert_max || ...
        lower_y < vert_min
    me = MException('spm:cut:rectangle_out_of_range', ...
        'Selected rectangle is outside channel frame.');
    throw(me);
end

% Define cut-out rectangle
extent_x = in.axis(1).variable.extent;
extent_y = in.axis(2).variable.extent;
[ny, nx] = size(in.pixels);
ix =  out.axis(1).variable.ref_index;
iy =  out.axis(2).variable.ref_index;
scale_x =  out.parameter(ix).display_scale;
scale_y =  out.parameter(iy).display_scale;

if extent_x == 0 || extent_y == 0
    me = MException('spm:cut:zero_extent', ...
        'Zero extent (is not supported).');
    throw(me);
end
if snap_to_grid && ~strcmp(axis_type,'none')
    lower_x = fix(1 + (nx - 1) * ...
        (lower_x - hor_min) / (abs(extent_x) * scale_x));
    lower_y = fix(1 + (ny - 1) * ...
        (lower_y - vert_min) / (abs(extent_y) * scale_y));
    upper_x = fix(1 + (nx - 1) * ...
        (upper_x - hor_min) / (abs(extent_x) * scale_x));
    upper_y = fix(1 + (ny - 1) * ...
        (upper_y - vert_min) / (abs(extent_y) * scale_y));
    axis_type = 'none';
    hor_min = 1;
    vert_min = 1;
    hor_max = size(in.pixels,2);
    vert_max = size(in.pixels,1);
end
rect = [lower_x, lower_y, upper_x - lower_x, upper_y - lower_y];

% Carry out the cut
out.pixels = imcrop([hor_min, hor_max], ...
    [vert_min, vert_max], in.pixels, rect);

% Adapt the axis limits and pixel counts
out.axis(1).count = size(out.pixels, 2);
out.axis(2).count = size(out.pixels, 1);

if strcmp(axis_type, 'none') % Pixel coordinates
    if nx > 1 % Otherwise something strange is going on, but
        % we are allowing empty channels for now
        if extent_x >= 0
            out.parameter(ix).start_value.fine =  ...
                in.parameter(ix).start_value.fine + extent_x *...
                (lower_x - 1)/ (nx - 1);
            out.parameter(ix).start_value.real =  ...
                in.parameter(ix).start_value.real + extent_x *...
                (lower_x - 1)/ (nx - 1);
        else
            out.parameter(ix).start_value.fine =  ...
                in.parameter(ix).start_value.fine + extent_x *...
                (nx - upper_x)/ (nx - 1);
            out.parameter(ix).start_value.real =  ...
                in.parameter(ix).start_value.real + extent_x *...
                (nx - upper_x)/ (nx - 1);
        end
        out.axis(1).variable.extent =  out.axis(1).variable.extent * ...
            (size(out.pixels, 2) - 1) / (nx - 1);
    end
    
    if ny > 1 % Otherwise something strange is going on, but
        % we are allowing empty channels for now
        if extent_y >= 0
            out.parameter(iy).start_value.fine =  ...
                in.parameter(iy).start_value.fine + extent_y *...
                (lower_y - 1)/ (ny - 1);
            out.parameter(iy).start_value.real =  ...
                in.parameter(iy).start_value.real + extent_y *...
                (lower_y - 1)/ (ny - 1);
        else
            out.parameter(iy).start_value.fine =  ...
                in.parameter(iy).start_value.fine + extent_y *...
                (ny - upper_y)/ (ny - 1);
            out.parameter(iy).start_value.real =  ...
                in.parameter(iy).start_value.real + extent_y *...
                (ny - upper_y)/ (ny - 1);
        end
        out.axis(2).variable.extent =  in.axis(2).variable.extent * ...
            (size(out.pixels, 1) - 1) / (ny - 1);
    end
else % Real-world coordinates
    
    if scale_x > 0 && scale_y > 0
        if extent_x >=0
            out.parameter(ix).start_value.fine = ...
                in.parameter(ix).start_value.fine + ...
                (lower_x - hor_min) / scale_x ;
            out.parameter(ix).start_value.real = ...
                in.parameter(ix).start_value.real + ...
                (lower_x - hor_min) / scale_x ;
            out.axis(1).variable.extent = rect(3) / scale_x;
        else
            out.parameter(ix).start_value.fine = ...
                in.parameter(ix).start_value.fine - ...
                (hor_max - upper_x) / scale_x ;
            out.axis(1).variable.extent = -rect(3) / scale_x;
        end
        
        if extent_y >=0
            out.parameter(iy).start_value.fine = ...
                in.parameter(iy).start_value.fine + ...
                (lower_y - vert_min) / scale_y ;
              out.parameter(iy).start_value.real = ...
                in.parameter(iy).start_value.real + ...
                (lower_y - vert_min) / scale_y ;
            out.axis(2).variable.extent = rect(4) / scale_y;
        else
            out.parameter(iy).start_value.fine = ...
                in.parameter(iy).start_value.fine - ...
                (vert_max - upper_y) / scale_y ;
            out.axis(2).variable.extent = -rect(4) / scale_y;
        end
    else
        me = MException('spm:cut:invalid_display_scale', ...
            'Negative display scales are not allowed.');
        throw(me);
    end
    
end

end

