function options = display_options(varargin)
%DISPLAY_OPTIONS Set display options.
%   DISPLAY_OPTIONS(INPUTFILE) returns a structure with the display options
%   as defined in the XML file INPUTFILE. DISPLAY_OPTIONS() opens a file 
%   dialogue to select the XML file and subsequently opens a GUI where the
%   loaded values can be edited. If the file dialogue is canceled, the GUI 
%   is opened with default values, which can be edited. Upon closing the 
%   GUI, the indicated values are returned in a structure. 
%
%   The indicated values can also be saved to an XML file by using the 
%   'Save as XML...' button at the top left corner of the screen. To write 
%   a certain (manually edited) options structure to file, call 
%   DISPLAY_OPTIONS(OPTIONS, OUTPUTFILE)  for an immediate save or 
%   DISPLAY_OPTIONS(OPTIONS) to invoke the aforementioned GUI.
%
%   For an explanation of the various options, the user is referred to
%   spm_display.

full_path = mfilename('fullpath');
root_path = fileparts(full_path);
default_xml_path = [root_path, filesep, 'xml', filesep];
xsd = [default_xml_path, 'display_options.xsd'];

if verLessThan('matlab', '7.14')
    error(nargchk(0, 2, nargin));
else
    narginchk(0, 2);
end
% No arguments
if nargin == 0 
    [xml_file, xml_path] = uigetfile([default_xml_path, '*.xml'], ...
        'Pick an XML display options file');
    if xml_file ~= 0
        xml = [xml_path, xml_file];
    else
        xml =  [default_xml_path, 'display_options.xml'];
    end
    options = set_options(xsd, xml);
 % One argument
elseif nargin == 1 
    % Argument is filename
    if ischar(varargin{1}) && exist(varargin{1}, 'file')
        xml = tilde_expand(varargin{1});
        %options = set_options(xsd, xml);
        options = xml_read(xml);
        options = rmfield(options, 'ATTRIBUTE');
    % Argument is structure
    elseif isstruct(varargin{1})
        xml = [tempdir, 'temp.xml'];
        xml_write(xml, varargin{1});
        options = set_options(xsd, xml);
    % Argument is something else
    else
        me = MException('spm:display_options:unknown_first_argument', ...
            'First argument unsupported. See "help display_options".');
        throw(me);
    end
% More than one argument, first argument is structure
elseif isstruct(varargin{1}) 
    output_file = fopen(varargin{2}, 'w');
    % Second argument is file that can be opened
    if output_file ~= -1
        fclose(output_file);
        xml_write(varargin{2}, varargin{1});
    % Second argument is something else
    else
        me = MException('spm:display_options:cannot_write_output', ...
            'Unable to open output file');
        throw(me);
    end
% Any other combination of inputs
else
    me = MException('spm:display_options:unsupported_input', ...
         'Unsupported input. See "help display_options".');
    throw(me);
end     



