function out = exp_tf(in, alpha)
%EXP_TF Construct an exponential channel.
%   EXP_TF(IN, ALPHA) returns an exponential channel with the same 
%   dimensions as  the input channel IN using the exponential factor ALPHA. 
%   Most of the channel fields are copied from IN. Only the channel 
%   (display) unit is set to '1' and the channel display scale is set to 1.
%   In addition, the pixel domain is set to 'frequency', as the output is
%   thought to be a filter applied in the frequency domain.

% Check input
try
    parser = inputParser;
    parser.addRequired('in', @(in) is_channel(in) && ...
        length(in.axis) == 2 && ~isempty(in.pixels));
    parser.addRequired('alpha', @isfloat);
    parser.parse(in,alpha);
catch error
    me = MException('spm:exp_tf:input_error', 'Invalid input: %s', ...
        error.message);
    throw(me);
end

% Copy in to out
out = in;

% Set pixels
out.pixels = ones(size(out.pixels));

% Compute k-matrix
dx = compute_dxdy(in, 1);
dy = compute_dxdy(in, 2);
[k, ~, ~] = generate_k_matrices(in, dx, dy);

%  Distance loss:
out.pixels = complex(exp(-alpha * k), 0);

% Set channel units
out.unit = '1';
out.display_unit = '1';
out.display_scale = 1;

% Set pixel domain
out.pixel_domain = 'frequency';

end