function window = construct_window(nofrows, nofcolumns, type, code, ...
    cutoffX, cutoffY)
%CONSTRUCT_WINDOW Helper function to construct lp/hp filtering window
%   CONSTRUCT_WINDOW(nofrows,nofcolumns,type,code,cutoffX,cutoffY) returns 
%   the window and cut limit to be used in used in lp_filter, hp_filter and
%    lphp_filter, where the parameters have the following meaning:
%   NOFROWS          Number of rows in the image to be filtered
%   NOFCOLUMNS       Number of columns in the image to be filtered
%   TYPE             Low-pass ('lp') or high-pass ('hp')
%   CODE             Code number for window type:
%                     0 = ideal with absolute cutoff
%                    +n = Butterworth of order n
%                    -n = exponential of order n
%   CUTOFFX          Cut-off frequency in x-direction in pixel coordinates
%   CUTOFFY          Cut-off frequency in y-direction in pixel coordinates
%                    This parameter is optional; by default it equals
%                    CUTOFFX.

if nargin == 5
    cutoffY = cutoffX;
end
if nofrows == 1
    nn = nofcolumns;
    window = [0:floor(nn/2)-1, nn-floor(nn/2)-(0:nn-floor(nn/2)-1)];
    if strcmp(type,'hp')
        %Special treatment of corners
        window(1,1) = 1;
        window(1,nofcolumns) = 1;
    end
elseif nofrows > 1
    nx = nofcolumns;
    ny = nofrows;
    cut_ratio = cutoffX /cutoffY;
    
    %Calculate frequency distance index in x and scaled index in y.
    dx = [0:floor(nx/2)-1, nx-floor(nx/2)-(0:nx-floor(nx/2)-1)];
    dy = cut_ratio*...
        [0:floor(ny/2)-1, ny-floor(ny/2)-(0:ny-floor(ny/2)-1)];
    
    %Square x and y components and create complete matrices.
    dx = dx .* dx;
    dy = dy .* dy;
    dx = ones(ny, 1) * dx;
    dy = dy' * ones(1, nx) ;
    
    %Set window to the frequency distance
    window = sqrt(dx+dy);
    
    if strcmp(type,'hp')
        %Special treatment of corners
        window(1,1) = 1;
        window(nofrows,1) = 1;
        window(1,nofcolumns) = 1;
        window(nofrows,nofcolumns) = 1;
    end
else
    fprint('Warning: nofrows < 1. Returning "all ones" matrix.');
    window = ones(nofrows,nofcolumns);
end

if strcmp(type,'hp')
    if code < 0 
        window	= exp(-(cutoffX*ones(size(window))./window).^abs(code));
    elseif code == 0 
        window	= (cutoffX < window);
    elseif code > 0 
        window	= ones(size(window)) ./ (ones(size(window))+...
            (cutoffX*ones(size(window))./window).^(2*code));
    end
    window(1,1) = 0;
    window(nofrows,1) = 0;
    window(1,nofcolumns) = 0;
    window(nofrows,nofcolumns) = 0;
else
    if code < 0 
        window = exp(-(window/cutoffX).^abs(code));
    elseif code == 0 
        window	= (window < cutoffX);
    elseif code > 0 
        window	= ones(size(window)) ./ (ones(size(window))+...
            (window/cutoffX).^(2*code));
    end
end
end
