function merged = dom_merge(dom1, dom2, config, format)
%DOM_MERGE Merge of two DOM trees
%   DOM_MERGE(DOM1, DOM2, CONFIG FORMAT) returns the result of merging the
%   DOM tree DOM1 into the DOM tree DOM2, according to rules specified
%   in the configuration file CONFIG. If FORMAT = 'dom', the result is a  
%   DOMtree; if FORMAT = 'string', the result is a string. By default, 
%   FORMAT  = 'dom'. 
%   To perform the merge, GeoNetwork Tv's version of the ELCA XMLMerge
%   library is used (see http://geonetwork.tv/xmlmerge/). The syntax of 
%   the configuration file is explained on pp. 213-214 of
%   http://el4j.sourceforge.net/docs/pdf/ReferenceDoc.pdf.

%Check innput
if nargin < 3
     me = MException('spm:dom_merge:too_few_arguments', ...
                           'Too few input arguments provided.');
     throw(me);
elseif (~isa(dom1, 'org.apache.xerces.dom.DocumentImpl') &&...
    ~isa(dom1, 'org.apache.xerces.dom.DeferredDocumentImpl'))
     ME = MException('spm:dom_merge:invalid_first_dom', ...
                           'First argument should be a DOM tree');
     throw(ME);
elseif (~isa(dom2, 'org.apache.xerces.dom.DocumentImpl') &&...
    ~isa(dom2, 'org.apache.xerces.dom.DeferredDocumentImpl'))
     ME = MException('spm:dom_merge:invalid_second_dom', ...
                           'Second argument should be a DOM tree');
     throw(ME);
elseif (~exist(config, 'file'))
     ME = MException('spm:dom_merge:invalid_file_name', ...
                           'Third argument should be a file name');
     throw(ME);
elseif nargin == 4 && ~strcmp(format, 'dom') && ~strcmp(format, 'string')
    ME = MException('spm:dom_merge:invalid_format', ...
                           'Format should be "dom" or "string"');
    throw(ME);
elseif nargin < 4
    format = 'dom';
else
    if nargin > 4
        warning('spm:dom_merge:extra_arguments_provided', ...
        'Only 4 input arguments needed. Ignoring extra arguments');
    end
end
config = get_full_path(config);

% Import relevant Java classes
import ch.elca.el4j.services.xmlmerge.config.*
import java.util.Properties;
import java.io.FileInputStream;

% Convert doms to xml strings
sources = {xmlwrite_xerces(dom1), xmlwrite_xerces(dom2)};

% Apply XMLmerge library to perform the merge
%properties_path = 'C:\Temp\el4j-xmlmerge-standalone-1.1.1\test.properties';
props = Properties();
fis = FileInputStream(config);
c = onCleanup(...
     @()fis.close());
try
    props.load(fis);    
catch e
    fprintf(e.message());
    ME = MException('spm:dom_merge:config_load_failed', ...
                           'Unable to load config-file');
    throw(ME);
end
configurer = PropertyXPathConfigurer(props);
xmlMerge= ConfigurableXmlMerge(configurer);
try 
	result = xmlMerge.merge(sources);
catch e
    fprintf(e.message);
    ME = MException('spm:dom_merge:merge_failed', ...
                           'Cannot merge trees.');
    throw(ME);
end				

% Convert output to DOM tree if desired, otherwise output directly 
if strcmp(format, 'dom')
    fid = fopen([tempdir, 'tempdom.xml'], 'w');
    fprintf(fid, '%s', char(result));
    fclose(fid);
    merged = xmlread([tempdir, 'tempdom.xml']);
    delete([tempdir, 'tempdom.xml']);
else
    merged = result;
end

end

