function [native_options, plot_options] = get_display_options(opt_args, ...
     dimension)
% GET_DISPLAY_OPTIONS Helper function for spm_display to read options 
%   [NATIVE_OPTIONS, PLOT_OPTIONS] = GET_DISPLAY_OPTIONS(OPT_ARGS, 
%   CHANNEL_TYPE) takes the spm_display optional inputs OPT_ARGS and 
%   transforms those to a structure of native options NATIVE_OPTIONS 
%   (these are specific to spm_display) and a structure of plot options 
%   PLOT_OPTIONS borrowed from Matlab's built-in plot command. The channel 
%   type CHANNEL_TYPE and the dimension DIMENSION are required as input 
%   arguments as the default options depend upon it.

% Read native option from file containing defaults
full_path = mfilename('fullpath');
root_path = fileparts(full_path);
default_xml_path = [root_path, filesep, '..', filesep, 'xml', filesep];
xml = [default_xml_path, 'display_options.xml']; % Default options
native_options = xml_read(xml);

% Initially plot options are empty
plot_options = struct();

% Override defaults with user options provided at the command line
% Options provided in a structure
if length(opt_args) == 1
    user_options = opt_args{1};
    if ~isstruct(user_options) 
        me = MException('spm:spm_display:no_options_provided', ...
            'First optional argument should be an options structure.');
        throw(me);
    end

    user_fields = fieldnames(user_options);
    for i=1:length(user_fields)
        if isfield(native_options, user_fields{i})
            native_options.(user_fields{i}) = ...
                user_options.(user_fields{i});
        else
            plot_options.(user_fields{i}) = user_options.(user_fields{i}); 
            % TODO: check sanity of this later on
        end
    end
    % Options provided as property-value pairs
else
    if mod(length(opt_args), 2) == 1
        me = MException('spm:spm_display:incomplete_option_spec', ...
            'Missing option value.');
        throw(me);
    else
        for i = 1:2:length(opt_args)
            if ischar(opt_args{i})
           
                if isfield(native_options, opt_args{i})
                    native_options.(opt_args{i}) = opt_args{i+1};
                else
                    plot_options.(opt_args{i}) = opt_args{i+1}; 
                    % To check sanity of this later on
                end
            else
                me = MException('spm:spm_display:invalid_option_name', ...
                'Option name is expected to be a string.');
            throw(me);  
            end
        end
    end
end


% Set automatic plot type
if strcmp(native_options.plot_type, 'automatic')
    if dimension > 1
        native_options.plot_type = 'image';
    else
        native_options.plot_type = 'line';
    end
end

% Set automatic color lut
if strcmp(native_options.color_lut, 'automatic')
    if  strcmp(native_options.plot_type, 'line')
        native_options.color_lut = 'minimal';
    else
        native_options.color_lut = 'gray';
    end
end


end

