function tf_out = mfm_tf(nx, dx, ny, dy, tf_in, derivative, ...
    transform, z, A, theta, phi, d, varargin)
% MFM_TF Transfer function calculation for MFM simulation
%   MFM_TF(NX, DX, NY, DY, TF_IN, DERIVATIVE, TRANSFORM, Z, A, THETA, PHI,
%   D, DELTAW) is the MFM transfer function where the input variables are
%   defined as follows:
%
%   NX, NY        Number of pixels in x, y-direction of the image for
%                 which the transfer function is to be calculated
%   DX, DY        Size in [m] of one pixel in x, y-direction
%   TF_IN         Channel containing a transfer function, eg. sigma_tip
%                 or the instrument calibration function (in Fourier
%                 space).
%   DERIVATIVE    0: No change in units
%                 1: Calculate z-derivative
%   TRANSFORM     0: No coordinate transform
%                 1: Convert z-component to n-component (tf*LCF)
%                 2: Convert z-component to n-derivative
%                    (tf*LCF*LCF)
%
%                 NOTE: use TRANSFORM = 2 if
%                       (DERIVATIVE = 1 AND transformation) is wanted
%
% -------------------------------------------------------------------------
% NOTE: the units of all the below parameters are assumed to be the same
%       as those of the IN pattern.
% -------------------------------------------------------------------------
%   Z             Height distance between the data in
%                 from_channel to the data in the tf_channel,
%                 in case 1. this would be the distance to the upper
%                 sample surface.
%
%   A             Vibration amplitude, set A = 0 to use
%                 the small amplitude approximation.
%
%                 NOTE: IF A NOT= 0 THEN use z = tip-sample distance + A !
%
%
%   THETA         presently 12 deg is implemented !
%   PHI           presently tilt is along y-axis (pointing from the top of
%                 the image towards the bottom)
%
%   D             D = D:
%                 a magnetization pattern of a sample with a thickness d
%                 is considered. TF = M 0.5 exp(-kz) [1-exp(-kd)] ...
%
%                 D = Inf
%                 a magnetization pattern of a sample with infinite
%                 thickness is considered. TF = M 0.5 exp(-kz) ...
%
%                 D = 0
%                 an infinitesimally thin sheet of a magnetic moment per
%                 area pattern is considered. TF = m/A 0.5 exp(-kz) ...
%
%                 D = NaN
%                 use if no boundary condition exists, i.e. when NOT
%                 starting from a M or m/A pattern.
%                 TF = input_field exp(-kz)
%
%   DELTA_W       domain wall width (optional, NaN-> not taken into
%                 account)

% Check input
try
    parser = inputParser;
    parser.addRequired('nx', @(nx)isnumeric(nx) && mod(nx, 1) == 0);
    parser.addRequired('dx', @isfloat);
    parser.addRequired('ny', @(ny)isnumeric(ny) && mod(ny, 1) == 0);
    parser.addRequired('dy', @isfloat);
    parser.addRequired('tf_in', @(tf_in)is_channel(tf_in) && ...
        strcmp(tf_in.type,'imageScan') && ...
        strcmp(tf_in.pixel_domain, 'frequency') && ~isempty(tf_in.pixels));
    parser.addRequired('derivative', @(derivative) derivative == 0 ||...
        derivative == 1);
    parser.addRequired('transform', @(transform) transform == 0 || ...
        transform == 1 ||  transform == 2);
    parser.addRequired('Z', @isfloat);
    parser.addRequired('A', @isfloat);
    parser.addRequired('theta', @isfloat);
    parser.addRequired('phi', @isfloat);
    parser.addRequired('d', @(d)(isfloat(d) && d >= 0) || isnan(d));
    parser.addOptional('delta_w', NaN, @(delta_w)(isfloat(delta_w) && ...
        delta_w >= 0) || isnan(delta_w));
    parser.parse(nx, dx, ny, dy, tf_in, derivative, transform, z, A, ...
        theta, phi, d, varargin{:});
    delta_w = parser.Results.delta_w;
catch error
    me = MException('spm:mfm_tf:input_error', 'Invalid input: %s', ...
        error.message);
    throw(me);
end

% Deal with zero domain wall width (-> no effect calculated)
if delta_w==0
    delta_w = NaN;
    warning('spm:mfm_tf:input_warning', ...
        ['Use of delta_w = 0 is deprecated. ', ...
        'Setting delta_w to NaN instead']);
end

% Initialize output channel
tf_out = tf_in;

% Initialize output channel
[k, kx, ky] = generate_k_matrices(tf_in, dx, dy);

%  Distance loss:
tf_out.pixels = complex(exp(-z * k), 0);

% Magnetization to stray field
% m/A to stray field for sheets of UCS chose d<0!
if ~isnan(d)
    tf_out.pixels = tf_out.pixels * 0.5;
    if (d ~= Inf)
        if (d ~= 0)
            tf_out.pixels = tf_out.pixels .* (1.0 - exp(-d * k));
        else
            tf_out.pixels = tf_out.pixels .* k;
            disp('UCS m/A calculated');
        end
    end
end

if (A == 0)
    %    Use small amplitude approximation:
    
    %    Effect of lever canting angle
    if (transform ~= 0)
        tf_out.pixels = tf_out.pixels .* complex(cos(theta), ...
            (cos(phi) * sin(-theta) * kx + sin(phi) * sin(-theta) * ky) ...
            ./ k).^transform;
    end
    %    Force or force gradient image?
    if (derivative == 1)
        tf_out.pixels = tf_out.pixels .* k;
    end
else
    if (transform == 0)
        min_n_nabla = complex(k, 0);
    else
        min_n_nabla = complex(cos(theta) * k, cos(phi) * sin(-theta) * ...
            kx + sin(phi) * sin(-theta) * ky);
    end
    
    if (derivative == 0)
        tf_out.pixels = tf_out.pixels .* (min_n_nabla ./ (A * k.^2))...
            .* cbesseli1(A * min_n_nabla) * 2;
    else tf_out.pixels = tf_out.pixels .* (min_n_nabla ./ (A * k))...
            .* cbesseli1(A * min_n_nabla) * 2;
    end
end

% Domain wall
if ~isnan(delta_w)
    tf_out.pixels = ...
        tf_out.pixels  .*  k .* delta_w/2 ./ sinh(k * (delta_w/2));
end

% Transfer function:
st = size(tf_in.pixels);
if(st(1) == ny && st(2) == nx)
    tf_out.pixels = tf_out.pixels .* tf_in.pixels;
else
    me = MException('spm:mfm_tf:transfer_function_wrong_size', ...
        'Transfer function wrong size !');
    throw(me);
end

end

