function channels = nanonis2channels(filename)
%NANONIS2CHANNELS Read a Nanonis file into an array of channels
%   NANONIS2CHANNELS(FILENAME) reads  the NANONIS file FILENAME into a
%   channel array and returns this array. The function is a helper function
%   for SPM_READ.

% Load file into Matlab using function provided by Nanonis
header = loadsxm(filename);
if isempty(header)
    me = MException('spm:nanonis2channels:unsupported_file', ...
        ['File type not supported. ', ...
        'Please provide a correct Nanonis file.']);
    throw(me);
end

% Analyze data info in header
lines_scan = textscan(header.data_info, '%s', 'Delimiter','\n');
lines = lines_scan{1};
nof_lines = size(lines,1);
start = 1;
while isempty(lines{start})
    start = start + 1;
end
names_scan = textscan(lines{start}, '%s', 'Delimiter','\t');
names = names_scan{1};

info = struct(names{1},cell(nof_lines-start,1));
for i = start + 1 : nof_lines
    line_scan =  textscan(lines{i}, '%s', 'Delimiter','\t');
    line = line_scan{1};
    for j = 1 : size(names, 1)
        info(i - start).(names{j}) = line{j};
    end
end

% Determine number of channels
channel_count = 0;
for n = 1 : size(info, 1)
    if ~isfield(info, 'Direction')
        me = MException('spm:nanonis2channels:unknown_channel_number', ...
            'Cannot determine number of channels.');
        throw(me);
    else
        if strcmp(info(n).Direction,'both')
            channel_count = channel_count + 2;
        else
            channel_count = channel_count + 1;
        end
    end
end
% channel_count = 2 * size(info, 1);

% Check presence of fields
if ~isfield(info, 'Name')
    me = MException('spm:nanonis2channels:unknown_channel_name', ...
        'Cannot determine channel names.');
    throw(me);
end

if ~isfield(info, 'Unit')
    me = MException('spm:nanonis2channels:unknown_channel_unit', ...
        'Cannot determine channel units.');
    throw(me);
end

if ~isfield(header, 'rec_date')
    me = MException('spm:nanonis2channels:unknown_recording_date', ...
        'Cannot determine recording date.');
    throw(me);
end

if ~isfield(header, 'scan_offset')
    me = MException('spm:nanonis2channels:unknown_scan_offset', ...
        'Cannot determine scan axes offsets.');
    throw(me);
end

if ~isfield(header, 'scan_pixels')
    me = MException('spm:nanonis2channels:unknown_scan_pixels', ...
        'Cannot determine number of pixels.');
    throw(me);
end

if ~isfield(header, 'scan_range')
    me = MException('spm:nanonis2channels:unknown_scan_range', ...
        'Cannot determine range of axes.');
    throw(me);
end

if ~isfield(header, 'scan_time')
    me = MException('spm:nanonis2channels:unknown_scan_time', ...
        'Cannot determine time to scan a line.');
    throw(me);
end

channels(channel_count) = Channel();
cc = 1;

for n = 1 : size(info, 1)
    % Misinterpretation of 'Direction'?
    if strcmp(info(n).Direction,'both')
        nof_subchannels = 2;
    else
        nof_subchannels = 1;
    end
    for j = 1 : nof_subchannels

        % Channel name
        channel.name = info(n).Name;
        % Channel unit
        channel.unit = info(n).Unit;
        % Channel display unit
        channel.display_unit = info(n).Unit;
        % Channel display scale
        channel.display_scale = 1;
        % Channel type
        channel.type = 'imageScan';
        % Channel context
        channel.context{1} = filename;
        % Channel session
        channel.session = '';
        % Context reference index
        channel.ref_index = -1;
        % Channel affine transform
        channel.affine_transform = [];
        % Channel comment
        channel.comment = '';
        % Date+time
        date = datestr(datenum(header.rec_date,'dd.mm.yyyy'), ...
            'yyyy-mm-dd');
        time = header.rec_time;
        channel.date_time = [date, 'T', time, '.000000Z'];
        % Control parameters
        channel.control = struct('name',{},'setting',{});
        % Parameter x
        channel.parameter(1,1).name = 'x';
        % Position
        channel.parameter(1,1).start_value.coarse = NaN;
        channel.parameter(1,1).start_value.fine = header.scan_offset(1);
        channel.parameter(1,1).start_value.real = NaN;
        % Drift
        channel.parameter(1,1).drift = NaN;
        % Shift
        channel.parameter(1,1).shift = NaN;
        % Unit
        channel.parameter(1,1).unit = 'm';
        % Display unit
        channel.parameter(1,1).display_unit = 'm';
        % Display scale
        channel.parameter(1,1).display_scale = 1;
        
        % Parameter x
        channel.parameter(2,1).name = 'y';
        % Position
        channel.parameter(2,1).start_value.coarse = NaN;
        channel.parameter(2,1).start_value.fine = header.scan_offset(1);
        channel.parameter(2,1).start_value.real = NaN;
        % Drift
        channel.parameter(2,1).drift = NaN;
        % Shift
        channel.parameter(2,1).shift = NaN;
        % Unit
        channel.parameter(2,1).unit = 'm';
        % Display unit
        channel.parameter(2,1).display_unit = 'm';
        % Display scale
        channel.parameter(2,1).display_scale = 1;
        
        % Parameter z_rel
        channel.parameter(3,1).name = 'z_rel';
        % Position
        channel.parameter(3,1).start_value.coarse = 0;
        channel.parameter(3,1).start_value.fine = 0;
        channel.parameter(3,1).start_value.real = 0;
        % Drift
        channel.parameter(3,1).drift = NaN;
        % Shift
        channel.parameter(3,1).shift = NaN;
        % Unit
        channel.parameter(3,1).unit =  'm';
        % Display unit
        channel.parameter(3,1).display_unit =  'm';
        % Display scale
        channel.parameter(3,1).display_scale = 1;
        
        % Horizontal axis:
        % Name
        channel.axis(1,1).name = 'x';
        % Count
        channel.axis(1,1).count = header.scan_pixels(1);
        % Direction
        if strcmp(info(n).Direction,'both')
            if mod(channel_count, 2) == 0
                channel.axis(1,1).direction = 'forward';
            else
                channel.axis(1,1).direction = 'backward';
            end
        else
             channel.axis(1,1).direction = info(n).Direction;
        end
        % Interval
        channel.axis(1,1).interval = NaN;
        % Variable
        channel.axis(1,1).variable.ref_index = 1;
        % Extent
        channel.axis(1,1).variable.extent = header.scan_range(1);
        
        % Vertical axis:
        % Name
        channel.axis(2,1).name = 'y';
        % Count
        channel.axis(2,1).count = header.scan_pixels(2);
        % Direction
        if strcmp(info(n).Direction,'both')
            if mod(channel_count, 2) == 0
                channel.axis(2,1).direction = 'forward';
            else
                channel.axis(2,1).direction = 'backward';
            end
        else
            channel.axis(2,1).direction = info(n).Direction;
        end
        % Interval
         channel.axis(2,1).interval = NaN;
%         if mod(channel_count, 2) == 1
%             channels(cc).axis(2,1).interval = header.scan_time(1);
%         else
%             channels(cc).axis(2,1).interval = header.scan_time(2);
%         end
        % Variable
        channel.axis(2,1).variable.ref_index = 2;
        % Extent
        channel.axis(2,1).variable.extent = header.scan_range(2);
        
        % Pixel space
        channel.pixel_domain = 'spatial';
        % Pixel type
        channel.pixel_type = 'real';
        % Pixels
        [~, data] = loadsxm(filename, cc - 1);
        if strcmp(info(n).Direction,'backward') || ...
                (strcmp(info(n).Direction,'both') && j == 1)
                channel.pixels = fliplr(data);
        else
             channel.pixels = data;
        end
        
        % Convert to Channel object
        channels(cc) = Channel(channel);
        cc = cc + 1;
    end
end


end

