function flag = same_domain(channel1, channel2)
%SAME_DOMAIN Check whether two imageScan channels have equal domains.
%   SAME_DOMAIN(CHANNEL1, CHANNEL2) returns true if and only if the  
%   functions represented by the imageScan channels CHANNEL1 and CHANNEL2 
%   have the same domain. If the real coordinates of the domain have not
%   been determined, it is assumed that there is zero shift.

% Check input
try
    parser = inputParser;
    parser.addRequired('channel1', @(channel1)is_channel(channel1) && ...
        strcmp(channel1.type,'imageScan'));
     parser.addRequired('channel2', @(channel1)is_channel(channel2) && ...
        strcmp(channel2.type,'imageScan'));
    parser.parse(channel1, channel2);
catch error
    me = MException('spm:same_domain:input_error', ...
        'Invalid input: %s', error.message);
    throw(me);
end

flag = false;

% Get axes and correposnding parameters
[hor_axis1, vert_axis1, hor_param1, vert_param1] = ...
    get_axes_and_params(channel1);
[hor_axis2, vert_axis2, hor_param2, vert_param2] = ...
    get_axes_and_params(channel2);

% Convert to S.I. if necessary
if ~strcmp(hor_param1.unit, 'm') || ~strcmp(vert_param1.unit, 'm')
    channel1 = convert2si(channel1);
    [hor_axis1, vert_axis1, hor_param1, vert_param1] = ...
    get_axes_and_params(channel1);
end
if ~strcmp(hor_param2.unit, 'm') || ~strcmp(vert_param2.unit, 'm')
    channel2 = convert2si(channel2);
    [hor_axis2, vert_axis2, hor_param2, vert_param2] = ...
    get_axes_and_params(channel2);
end


if ~isnan(hor_param1.start_value.real) && ...
        hor_param1.start_value.real ~= hor_param2.start_value.real
    return;
elseif ~isnan( vert_param1.start_value.real) && ...
        vert_param1.start_value.real ~= vert_param2.start_value.real
    return;
elseif isnan(hor_param1.start_value.real) && ...
        ~isnan(hor_param1.start_value.coarse) && ...
        hor_param1.start_value.fine + hor_param1.start_value.coarse ~= ...
     hor_param2.start_value.fine + hor_param2.start_value.coarse
    return;
elseif isnan(vert_param1.start_value.real)  && ...
        ~isnan(vert_param1.start_value.coarse) && ...
        vert_param1.start_value.fine + vert_param1.start_value.coarse ...
     ~= vert_param2.start_value.fine + vert_param2.start_value.coarse
    return;
elseif isnan(hor_param1.start_value.coarse) && ...
        hor_param1.start_value.fine ~= hor_param2.start_value.fine
    return;
elseif isnan(vert_param1.start_value.coarse) && ...
        vert_param1.start_value.fine ~= vert_param2.start_value.fine
    return;
elseif hor_axis1.variable.extent ~= hor_axis2.variable.extent;
    return;
elseif vert_axis1.variable.extent ~= vert_axis2.variable.extent;
    return;
elseif ~isequal(size(channel1.pixels), size(channel2.pixels))
    return;
else
    flag = true;
end
    
end

