function channel = image2channel(im, x_size, y_size, unit, domain)
%IMAGE2CHANNEL Image to (imageScan) channel conversion.
%   IMAGE2CHANNEL(IM, X_SIZE, Y_SIZE, UNIT, DOMAIN) creates an imageScan
%   channel with pixel matrix IM, containing an image  with  spatial 
%   dimensions X_SIZE by Y_SIZE measured in the unit UNIT and pixel
%   domain DOMAIN. In the spatial domain, the left lower corner of the 
%   image is (0,0). 
%
%   The display unit of both the channel axes is UNIT. The internal units 
%   are set to 'm' where possible. If no automatic conversion between UNIT 
%   and 'm' can be done, the internal unit is also UNIT. In this case, a 
%   warning is issued.

% Check input
try
    parser = inputParser;
    parser.addRequired('im', @(im)(isnumeric(im) && ~isempty(im)));
    parser.addRequired('x_size', @(x_size)(numel(x_size) == 1 && ...
        isnumeric(x_size) && x_size > 0));
    parser.addRequired('y_size', @(y_size)(numel(y_size) == 1 && ...
        isnumeric(y_size) && y_size > 0));
    parser.addRequired('display_unit', @ischar);
    parser.addRequired('domain', @(domain)(strcmp(domain,'spatial') || ...
        strcmp(domain,'frequency')));
    parser.parse(im, x_size, y_size, unit, domain);
catch error
    me = MException('spm:image2channel:input_error', ...
        'Invalid input: %s', error.message);
    throw(me);
end

% Construct channel

% Channel name
channel.name = '?';

% Channel unit
channel.unit = '1';
% Channel display unit
channel.display_unit = '1';
% Channel display scale
channel.display_scale = 1;
% Channel type
channel.type = 'imageScan';
% Channel context
channel.context = {};
% Context reference index
channel.ref_index = -1;
% Affine transform
channel.affine_transform = [];
% Context reference index
channel.comment = '';
% Data+time
channel.date_time = '0001-01-01T00:00:00.000000Z';
% Parameter corresponding to horizontal axis:
% Name
channel.parameter(1,1).name = 'x';
% Position
channel.parameter(1,1).start_value.coarse = 0;
channel.parameter(1,1).start_value.fine = 0;
channel.parameter(1,1).start_value.real = 0;
% Drift
channel.parameter(1,1).drift = 0;
% Shift
channel.parameter(1,1).shift = NaN;
% Unit
channel.parameter(1,1).unit =  unit;
% Display unit
channel.parameter(1,1).display_unit = unit;
% Display scale
channel.parameter(1,1).display_scale = 1;

% Parameter corresponding to vertical axis
% Name
channel.parameter(2,1).name = 'y';
% Position
channel.parameter(2,1).start_value.coarse = 0;
channel.parameter(2,1).start_value.fine = 0;
channel.parameter(2,1).start_value.real = 0;
% Drift
channel.parameter(2,1).drift = 0;
% Shift
channel.parameter(2,1).shift = NaN;
% Unit
channel.parameter(2,1).unit =  unit;
% Display unit
channel.parameter(2,1).display_unit = unit;
% Display scale
channel.parameter(2,1).display_scale = 1;

% Parameter corresponding to relative z-position
channel.parameter(3,1).name = 'z_rel';
% Position
channel.parameter(3,1).start_value.coarse = 0;
channel.parameter(3,1).start_value.fine = 0;
channel.parameter(3,1).start_value.real = 0;
% Drift
channel.parameter(3,1).drift = NaN;
% Shift
channel.parameter(3,1).shift = NaN;
% Unit
channel.parameter(3,1).unit =  unit;
% Display unit
channel.parameter(3,1).display_unit = unit;
% Display scale
channel.parameter(3,1).display_scale = 1;

% Horizontal axis:
% Name
channel.axis(1,1).name = 'x';
% Count
channel.axis(1,1).count = size(im, 2);
% Direction
channel.axis(1,1).direction = 'forward';
% Interval
channel.axis(1,1).interval = 0;
% Variable
channel.axis(1,1).variable.ref_index = 1;
channel.axis(1,1).variable.extent = x_size;

% Vertical axis:
% Name
channel.axis(2,1).name = 'y';
% Count
channel.axis(2,1).count = size(im, 1);
% Direction
channel.axis(2,1).direction = 'forward';
% Interval
channel.axis(2,1).interval = 0;
% Variable
channel.axis(2,1).variable.ref_index = 2;
channel.axis(2,1).variable.extent = y_size;

% Pixel space
channel.pixel_domain = domain;
%Pixel type
if isreal(im)
    channel.pixel_type = 'real';
else
    channel.pixel_type = 'complex';
end
% Pixels
channel.pixels = im;

% Convert struct to class object
channel = Channel(channel);

% Convert axes units to S.I. (if possible)
if ~strcmp(unit,'m')
    channel = convert2si(channel);
end

end

