function [decon, filter] = mfm_inv_calc_flat(in, z, thickness, delta_w, ...
    amplitude, derivative, transform, tf, method, parameter)
%MFM_INV_CALC_FLAT Function for inverse MFM calculations. 
%   [DECON, FILTER] = MFM_INV_CALC_FLAT(IN, Z, THICKNESS, DELTA_W, 
%   AMPLITUDE, DERIVATIVE,TRANSFORM, TF, METHOD, PARAMETER) computes a  
%   deconvolution result (e.g. a stray field from frequency shift 
%   measurement). The output of the function is the deconvolved channel 
%   (DECON) and the deconvolution filter (FILTER). The input parameters
%   are:
%
%   IN             Spatial channel of type imageScan that contains the
%                  originally measured MFM data.
%
%   Z              Height distance in nm from the input data to result data
%                  when the transfer function is applied. When the
%                  deconvolution result should be a magnetization pattern,
%                  this should be the distance to the top of the magnetic
%                  layer.
%
%   THICKNESS      If the transfer function should not be used to
%                  convert a magnetization pattern to stray-field,
%                  set THICKNESS = Inf.
%                  In other cases
%                  THICKNESS > 0: magnetic sample thickness [m]
%                  THICKNESS = 0: magnetic sample has infinitesimally small
%                                 thickness.
%
%   DELTA_W        If the deconvolution result should not be a
%                  magnetization pattern, then set DELTA_W = 0.
%                  Otherwise: domain wall width in nm, according to
%                  definition:
%                  DELTA_W = pi * sqrt(A/K),
%                  with A the exchange constant and K the uniaxial
%                  anisotropy constant.
%
%   AMPLITUDE      Vibration amplitude in nm, set A0 to zero to use the
%                  small amplitude approximation.
%
%   DERIVATIVE     0: no change in units
%                  1: remove z-derivative
%
%   TRANSFORM      0: no coordinate transform
%                  1: convert n-component to z-component
%                  2: convert n-derivative to z-derivative the
%
%   TF             Channel of type imageScan that contains the instrument
%                  calibration function of the tip, in frequency domain.
%
%   METHOD         Number that determines the deconvolution procedure that
%                  is used and the interpretation of the parameter array,
%				   in Matlab notation (e.g. [1,2,3,4] for method 3):
%
%                  0: Division,
%                        parameter(1): upper limit to the transfer
%                        function, a negative value means no limit
%                        parameter(2): lower limit to the transfer
%                        function, a negative value means no limit
%
%  		           1: Tikonov method,
%                        parameter(1): mu parameter for image restoration
%                        low mu results in much restoration and noise, high
%                        mu gives little restoration and little additional
%                        noise.
%
%                  2: Projected Landweber method,
%                        parameter(1): theta parameter
%                        parameter(2): lower bound to the reconstructed
%                        image
%                        parameter(3): upper bound to the reconstructed
%                        image
%                        parameter(4): number of iterations
%
%  		           3: Conjugate gradient method,
%                        parameter(1): number of iterations
%
%   PARAMETER      Array containing METHOD options. See the explanation
%                  of METHOD above for more details.
%
%   The function is based on a theory described in J. Appl. Phys.  88 (1),
%   p. 435 (2000), Hug et al., J. Appl. Phys. 83, p. 5609 (1998), and
%   Bertero, 'Inverse problems in imaging', IOP press, London, 1998
%
%   Change history original PV-WAVE file:
%   Version 4.5,   14.6.2001
%
%   PvS, 23.5.2001: Fixed unit conversion bug (nm sized measurements were
%   not handeled correctly)
%   Pvs,PK 14.06.2001: Fixed numerical problem in Tikonov method.

% Check input
try
    parser = inputParser;
    parser.addRequired('in', @(in)is_channel(in) && ...
        strcmp(in.type,'imageScan') && ...
        strcmp(in.pixel_domain, 'spatial') && ...
        ~isempty(in.pixels));
    parser.addRequired('z', @isfloat);
    parser.addRequired('thickness', ...
        @(thickness)(isfloat(thickness) && thickness >= 0) || ...
        isnan(thickness));
    parser.addRequired('delta_w', @(delta_w)(isfloat(delta_w) && ...
        delta_w >= 0) || isnan(delta_w));
    parser.addRequired('amplitude', @isfloat);
    parser.addRequired('derivative', @(derivative) derivative == 0 ||...
        derivative == 1);
    parser.addRequired('transform', @(transform) transform == 0 || ...
        transform == 1 ||  transform == 2);
    parser.addRequired('tf', @(tf)is_channel(tf) && ...
        strcmp(tf.type,'imageScan') && ...
        strcmp(tf.pixel_domain, 'frequency')  && ~isempty(tf.pixels));
    parser.addRequired('method', @(method)any(method == [0,1,2,3]));
    parser.addRequired('parameter', @(parameter)isnumeric(parameter) && ...
        ~isempty(parameter) && length(parameter) <= 4);
    parser.parse(in, z, thickness, delta_w, amplitude, derivative, ...
        transform, tf, method, parameter);
catch error
    me = MException('spm:mfm_inv_calc_flat:input_error', ...
        'Invalid input: %s', error.message);
    throw(me);
end

% Check coherence between parameter and method
method_names = {'Division', 'Tikonov', 'Projcted Landweber', ...
    'Conjgate Gradient'};
nof_params = [2,1,4,1];
if length(parameter) ~= nof_params(method + 1)
    me = MException('spm:mfm_inv_calc_flat:parameter_number_error', ...
        'Wrong number of parameters. Method %s uses %d parameters.', ...
        method_names{method + 1}, nof_params(method + 1));
    throw(me);
end

if (method == 2 && (floor(parameter(4)) ~= parameter(4) || ...
        parameter(4) < 1)) || ...
        (method == 3 && (floor(parameter(1)) ~= parameter(1) || ...
        parameter < 1))
    me = MException(...
        'spm:mfm_inv_calc_flat:false_number_of_iterations', ...
        'Number of iterations should be a positive integer.');
    throw(me);
end

% Constant definitions
% Angle between the (upward) lever normal and the z-axis. (0 - 90 degrees)
theta = (12.0 / 180.0) * pi ;
% Angle between the (upward) lever normal and the x-axis. (-180 - 180
% degrees)
phi = (-90.0 / 180.0) * pi ;

work_image = in.pixels();
s = size(work_image);
nx = s(2);
ny = s(1);

% Calculation of a Fourier transfer function that contains the distance and
% film thickness parts of the transfer function
tf = mfm_tf(nx, compute_dxdy(in, 1), ny, compute_dxdy(in, 2),...
    tf, derivative, transform, z, amplitude, theta, phi,...
    thickness, delta_w);
tf.pixels(1,1) = complex(real(mean(tf.pixels(:))), 0.0);

parsize = length(parameter);

switch  method
    case 0
        if parsize ~= 2
            printf('Not the right amount of parameters');
            printf('for division deconvolve!\n');
            return;
        end
        
        if (parameter(1) >= 0) && (parameter(1) <= max(abs(tf.pixels(:))))
            tf.pixels(abs(tf.pixels) > parameter(1)) = parameter(1);
        end
        if (parameter(2) >= 0) && (parameter(2) >=  max(abs(tf.pixels(:))))
            tf.pixels(abs(tf.pixels) < parameter(2)) = parameter(2);
        end
        
        work_image = fft2(work_image);
        work_image = real(ifft2(work_image ./ tf.pixels));
    case 1
        if parsize ~= 1
            printf('Not the right amount of parameters');
            printf('for Tikonov deconvolve!\n');
            return;
        end
        recon_tf = ones(size(tf.pixels))./...
            (tf.pixels+parameter(1)*ones(size(tf.pixels))./...
            conj(tf.pixels));
        % Assign the window function to tf
        tf.pixels = recon_tf .* tf.pixels;
        work_image = real(ifft2(fft2(work_image) .* recon_tf));
    case 2
        if parsize ~= 4
            printf('Not the right amount of parameters');
            printf('for projected Landweber deconvolve!\n');
            return;
        end
        tf2 = abs(tf.pixels).^2;
        tau = parameter(1) / max(tf2(:));
        tauKg = tau * conj(tf.pixels) .* fft2(work_image);
        tf.pixels = 1 - tau * tf2;
        work_image(:)= 0;
        for k = 1:parameter(4)
            % Deconvolution according to Landweber method
            work_image = fft2(work_image);
            work_image = tauKg + tf.pixels .* work_image;
            work_image = real(ifft2(work_image));
            
            % Projection on the constrained solution space
            work_image = min(max(work_image,parameter(1)),parameter(2));
        end
    case 3
        if parsize ~= 1
            printf('Not the right amount of parameters');
            printf('for conjugate gradient deconvolve!\n');
            return;
        end
        % Initialize the perpendicular vector:
        r = conj(tf.pixels) .* fft2(work_image);
        % Initialize the conjugate vector:
        p = r;
        % Initialize the result
        work_image(:)= 0;
        % Initialize some auxiliary variables
        tf2 = abs(tf.pixels).^2;
        new_int_r = sum(sum(abs(r).^2));
        
        for k = 1:parameter(1)
            % Deconvolution according to conjugate gradient method
            int_r = new_int_r;
            alpha = int_r / sum(sum(tf2 * abs(p).^2));
            r = r - alpha * tf2 .* p;
            new_int_r = sum(sum((abs(r).^2)));
            beta = new_int_r / int_r;
            work_image = work_image + alpha * p;
            p = r + beta * p;
            fprintf('iteration, alpha, beta    %f   %f   %f\n',...
                k, alpha, beta);
        end
        work_image = real(ifft2(work_image));
end

decon = in;
decon.pixels = work_image;

filter =  in;
filter.pixels = tf.pixels;

fprintf('Maximal amplification of the reconstruction filter: %f\n',...
    max(abs(tf.pixels(:))));

% Set z_level
if strcmp(decon.parameter(3).name, 'z_rel')
    decon.parameter(3).start_value.fine = ...
        decon.parameter(3).start_value.fine - z;
    decon.parameter(3).start_value.real = ...
        decon.parameter(3).start_value.real - z;
else
    me = MException('spm:mfm_inv_calc_flat:Structural_error', ...
        'Parameter z_rel missing in input channel');
    throw(me);
end

end

