function  spm_display(channel, varargin)
%SPM_DISPLAY Display channel data.
%   SPM_DISPLAY(CHANNEL) displays the data in CHANNEL, showing an image
%   when the data is 2D and of type imageScan and line plots when it is of
%   type generalScanStandard and/or 1D using default display options
%   spm_display(CHANNEL, OPTIONS) uses the display options as defined in 
%   the structure OPTIONS.
%
%   Allowed options are those listed below and/or (in case of line plots
%   with a 'minimal' color LUT). All options that are used in the Matlab
%   built-in 'plot' (see 'doc plot' for more information). Options can be
%   set throught the display_options GUI (type 'help display_options' for
%   more information), or by using the common property-value syntax, i.e.,
%   spm_display(channel, 'PropertyName', PropertyValue, ...).  If both an
%   options structure and a property-value pair are given to define a
%   property, then the latter prevails.
%
%   DISPLAY OPTIONS
%   -----------------------------------------------------------------------
%   'data_component': 'real', 'imaginary', 'magnitude', 'log_magnitude',
%   'phase' (default 'real') 
%   -----------------------------------------------------------------------
%   Data component to be displayed (especially relevant for complex data).
%
%
%   -----------------------------------------------------------------------
%   'scaling_factor': ... 0.25, 0.5, 1, 2, 3, 4, ... (default 1) 
%   -----------------------------------------------------------------------
%   Number of display pixels per data pixel.
%
%
%   -----------------------------------------------------------------------
%   'mode': 'raw', 'global', 'line' (default 'raw')
%   -----------------------------------------------------------------------
%   Processing type applied to channel before displaying:
%   'raw'      channel data is displayed without processing 
%   'global'   a best-fitting polynom of order 'order' is subtracted from 
%              the image
%   'line'     a best-fitting polynom of order 'order' is subtracted from 
%              each scan line.
%
%
%   -----------------------------------------------------------------------
%   'order': 0 or 1
%   -----------------------------------------------------------------------
%   Order of polynom to be subtracted before display (see explanation 
%   'mode').
%
%
%   -----------------------------------------------------------------------
%   'range_type': 'auto', 'norm', 'stdev', 'none' (default 'auto')
%   -----------------------------------------------------------------------
%   Type of channel range (z-scale) that is displayed; basically defines 
%   how the lower/upper channel value boundaries 'range_min' and 
%   'range_max' are interpreted:
%   'auto'      no truncation applied (range_min,range_max ignored)
%   'norm'      values are truncated to fit [range_min,range_max]
%   'stdev'     values are truncated to fit [mean + range_min * stddev, 
%               mean + range_max * stddev] 
%              (typically <range_min,range_max> = <-3,3>)
%   'none'      as 'auto', but with omission of the color bar; size of 
%               display window is tighter.
%
%
%   -----------------------------------------------------------------------
%   'range_min': lower channel value boundary
%   -----------------------------------------------------------------------
%   See explanation 'range_type'.
%
%
%   -----------------------------------------------------------------------
%   'range_max': upper channel value boundary
%   -----------------------------------------------------------------------
%   See explanation 'range_type'.
%
%
%   -----------------------------------------------------------------------
%   'iv1_axis': 'relative', 'center', 'fine', 'coarse', 'real_fine', 
%   'real_coarse', 'relative_real', 'none' (default 'relative')
%   -----------------------------------------------------------------------
%    Axis type to use in display for first dependent variable (iv1): 
%   'relative'      axis starts from 0, independent of its real start value 
%   'center'        center of axis is 0 
%   'fine'          axis starts with its real start value in fine 
%                   coordinates
%   'coarse'        axis starts with its real start value in coarse  
%                   + fine coordinates (i.e., the coarse position is added 
%                   as an offset)
%   'real_fine'     axis starts with its real start value in drift-
%                   corrected fine coordinates
%   'real_coarse'   axis starts with its real start value in drift-
%                   corrected coarse  + fine coordinates (i.e. , the coarse 
%                   position is added as an offset)
%   'relative_real' axis starts at 0; data is moved in such a way that the
%                   original (drift-corrected) left corner of the displayed
%                   sample is located at 0; missing data is filled in by 
%                   circular shifting.
%   'none'          axis will not be displayed (no ticks/labels); display 
%                   window will be correspondingly smaller.
%   Note that the options 'real_fine', 'real_coarse' and 'relative_real'
%   and 'relative_real' require the axis drift to be set to a valid number.
%   In addition, 'coarse' and 'real_coarse' depend on the coarse 
%   start_value of the axis parameter. Thus, if the drift and/or the coarse
%   value are NaN, these depending display options become invalid and
%   spm_display will throw an error.
%
%
%   -----------------------------------------------------------------------
%   'iv2_axis': 'relative', 'center', 'fine', 'coarse', 'real_fine', 
%   'real_coarse', 'relative_real', 'none' (default 'relative')
%   ----------------------------------------------------------------------
%    Axis type to use in display for first dependent variable (iv2). See 
%   'iv1-axis' for a further explanation.
%
%
%   -----------------------------------------------------------------------
%   'color_lut': 'automatic', 'jet', 'hsv', 'hot', 'cool', 'spring',
%   'summer', 'autumn', 'winter', 'gray', 'bone', 'copper', 'pink',
%   'lines', 'colorcube', 'flag', 'prism', 'white', 'minimal' (default
%   'automatic')
%   -----------------------------------------------------------------------
%   Color map used for displaying the channel pixel data. When applied to a 
%   line plot the color of the graph curve depends on its vertical 
%   position, that is, in general the graph changes color. Names refer to 
%   Matlab's built-in color maps (see 'help graph3d'), with the exception 
%   of:
%   'minimal'   black/white for image plots and no map for line plots
%   'auto'      color_lut 'gray' for images and 'minimal' for line plots.
%
%
%   -----------------------------------------------------------------------
%   'window_name': figure window name (default '')
%   -----------------------------------------------------------------------
%   The window displaying the channel is labeled with this string. If the 
%   provided string is empty, the name is set to the first argument of 
%   spm_display (i.e., the string representing IN).
%
%
%   -----------------------------------------------------------------------
%   'plot_type': 'automatic', 'image', 'line' (default 'automatic') 
%   -----------------------------------------------------------------------
%   Type of plot to display the image:
%   'automatic'     multiple row data -> image plot 
%                   single row data -> line plot
%   'image'         image plot (even if the data is effectively 1D)
%   'line'          (collection of) line plot(s) displaying each data row.

% Upon error exit
prefs.axis_visible = iptgetpref('ImshowAxesVisible');
prefs.border = iptgetpref('ImshowBorder');
prefs.magnification = iptgetpref('ImshowInitialMagnification');
c1 = onCleanup(...
    @()restore_iptprefs(prefs));

% Read and check input

% Number of inputs
if verLessThan('matlab', '7.14')
    error(nargchk(1, Inf, nargin)); %#ok<NCHKN>
else
    narginchk(1, Inf);
end

% Channel
if ~is_channel(channel)
    me = MException('spm:spm_display:no_channel_provided', ...
        'First argument should be a channel.');
    throw(me);
end

% Options: read defaults from file
[native_options, plot_options] = get_display_options(varargin, ...
    length(channel.axis));

% Create figure (and take care of clean-up in case of error)
window_name = char(native_options.window_name);
if isempty(window_name)
    window_name = inputname(1);
end
if isempty(window_name)
    function_call = call_string();
    function_call = strtrim(function_call);
    if function_call(end) == ';'
        function_call = function_call(1:end - 1);
    end
    function_call = strtrim(function_call);
    left_bracket = strfind(function_call,'(');
    
    if ~isempty(left_bracket)
        start = left_bracket(1) + 1;
        for i=start:length(function_call)
            if function_call(i) == ','
                try
                    evalin('caller', [function_call(start:i-1),';']);
                catch
                    continue;
                end
                break;
            end
        end
        change_name = true;
        if i == length(function_call)
            try
                evalin('caller', [function_call(start:i-1),';'])
            catch
                change_name = false;
            end
        end
        if change_name
            window_name = strtrim(function_call(start:i-1));
        end
    end
end

fig = figure('name', window_name);
set(fig, 'visible', 'off');

try
    % Get axes and (first) parameters
    [hor_axis, vert_axis, ~, ~] = get_axes_and_params(channel);
    
    % Compute pixels to be displayed
    channel.pixels = get_display_pixels(channel, native_options);
    
    % Compute display range
    valid = ~isnan(channel.pixels);
    if isempty(find(valid))
        warning('spm:spm_display:all_nan', ...
            'Channel only contains NaN values. Setting range to [0,0].');
        minimum = 0;
        maximum = 0;
    else
        minimum = min(channel.pixels(valid));
        maximum = max(channel.pixels(valid));
    end
    switch native_options.range_type
        case 'auto'
            lower = minimum;
            upper = maximum;
        case 'norm'
            lower = native_options.range_min/ ...
                channel.display_scale;
            upper = native_options.range_max/ ...
                channel.display_scale;
        case 'stdev'
            lower = mean(channel.pixels(valid)) + ...
                native_options.range_min * std(channel.pixels(valid));
            upper = mean(channel.pixels(valid)) + ...
                native_options.range_max * std(channel.pixels(valid));
        case 'none'
            lower =  minimum;
            upper = maximum;
        otherwise
            me = MException('spm:spm_display:unknown_range_type', ...
                'Unknown range type.');
            throw(me);
    end
    
    % To prevent lower = upper
    if lower == upper
        warning('spm:spm_display:empty_range', ...
            ['Calculated range is empty.',...
            ' Adding epsilon to cope with this.']);
        upper = lower + eps(single(lower)); % Double precision eps does not
        % work too well
    end
    
    if strcmp(native_options.plot_type, 'image') % Display as 2-D image
        % Set initial maginification to 100%
        iptsetpref('ImshowInitialMagnification', 100);
        
        % Set color map
        if strcmp(native_options.color_lut,'minimal')
            cmap = [repmat([0 0 0], 32, 1); repmat( [1 1 1],32, 1)];
        else
            cmap = native_options.color_lut;
        end
        
        if strcmp(native_options.iv1_axis, 'none') ||  ...
                strcmp(native_options.iv2_axis, 'none')
            % No axis
            iptsetpref('ImshowAxesVisible', 'off');
            im = imshow(channel.pixels,  'Colormap', colormap(cmap), ...
                'DisplayRange', [lower, upper]);
            % No border
            set(gca,'units','pixels')
            ax_pos = get(gca,'position');
            set(gcf,'units','pixels')
            fig_pos = get(gcf,'position');
            set(gcf,'position',[fig_pos(1) fig_pos(2) ...
                ax_pos(3) ax_pos(4)])
            set(gca,'units','normalized','position',[0 0 1 1])
            set(im, 'Tag', 'channel_image');
            set(gca,'Tag', 'channel_axes');
            set(fig, 'MenuBar', 'none', 'Toolbar', 'none');
        else
            iptsetpref('ImshowAxesVisible', 'on');
            [x_min, x_max, y_min, y_max] = get_axes_limits(channel, ...
                native_options.iv1_axis, native_options.iv2_axis);
            if ~isempty(find(isnan([x_min, x_max, y_min, y_max]), 1))
                me = ...
                    MException(...
                    'spm:spm_display:undefined_axis_limits', ...
                    'Axis limits are undefined.');
                throw(me);
            end
          
            % Display image
            im = imshow(channel.pixels, 'Colormap', colormap(cmap), ...
                'DisplayRange', [lower, upper]);
            
            % Set x-ticks
            x_lim = get(gca, 'XLim');
            if x_max > x_min
                s = get(gca);
                [xtick, xtick_label, x_scale_str] = ...
                    calcticks([x_min x_max], 'x');
                xtick = x_lim(1) * ones(size(xtick)) + ...
                    (x_lim(2) - x_lim(1)) / (x_max - x_min) * ...
                    (xtick - x_min  * ones(size(xtick)));
            else
                xtick = (x_lim(1) + x_lim(2)) / 2;
                if floor(x_min) == x_min
                    xtick_label = sprintf('%d', x_min);
                else
                    xtick_label = sprintf('%f', x_min);
                end
                x_scale_str = '';
            end
            xtick_label = cellstr(xtick_label);
            set(gca, 'XTick', xtick, 'XTickLabel', xtick_label);
            
            % Set y-ticks
            y_lim = get(gca, 'YLim');
            if y_max > y_min
                % Deliberately setting the direction to 'x' as it produced
                % nicer results
                [ytick, ytick_label, y_scale_str] = ...
                    calcticks([y_min y_max], 'x');
                ytick = y_lim(1) * ones(size(ytick)) + ...
                    (y_lim(2) - y_lim(1)) / (y_max - y_min) * ...
                    (ytick - y_min  * ones(size(ytick)));
            else
                ytick = (y_lim(1) + y_lim(2)) / 2;
                if floor(y_min) == y_min
                    ytick_label = sprintf('%d', y_min);
                else
                    ytick_label = sprintf('%f', y_min);
                end
                y_scale_str = '';
            end
            set(gca, 'YTick', ytick, 'YTickLabel', ytick_label);
            
            % Reverse  y-axis
             set(gca, 'YDir', 'normal');
                         
            % Tag image and axis
            set(im, 'Tag', 'channel_image');
            set(gca,'Tag', 'channel_axes');
            
            % Restore original axis position
            ax_units = get(gca, 'Units');
            set(gca, 'Units', 'pixels');
            original_position = get(gca, 'Position');
           
            % Set image axis labels
            [hor_unit, vert_unit, channel_unit] = get_display_units(...
                channel, native_options.data_component);
            [hor_name, vert_name, channel_name] = get_axes_names(...
                channel, native_options.data_component);
            if strcmp(channel.pixel_domain,'spatial')
                x_label = sprintf('%s [%s]', hor_name, ...
                    hor_unit);
                y_label = sprintf('%s [%s]', vert_name, ...
                    vert_unit);
            else
                x_label = sprintf('%s-Frequency [%s]', hor_axis.name, ...
                    hor_unit);
                y_label = sprintf('%s-Frequency [%s]', vert_axis.name, ...
                    vert_unit);
            end
            xlabel(x_label);
            ylabel(y_label);
            
            % Display scale string
            s = get(gca);
            text(x_lim(1),y_lim(2), x_scale_str, ...
                'HorizontalAlignment', 'left',...
                'VerticalAlignment', 'bottom', ...
                'FontAngle',s.FontAngle, ...
                'FontName', s.FontName, ...
                'FontSize',s.FontSize, ...
                'FontWeight',s.FontWeight);
            x_label_pos =  get(get(gca,'XLabel'),'Position');
            text(x_lim(2), x_label_pos(2), y_scale_str, ...
                'HorizontalAlignment', 'right',...
                'VerticalAlignment', 'top', ...
                'FontAngle',s.FontAngle, ...
                'FontName', s.FontName, ...
                'FontSize',s.FontSize, ...
                'FontWeight',s.FontWeight);
            
            if  ~strcmp(native_options.range_type, 'none')
                % Make sure that image is not rescaled and that figure is
                % to add color bar
                fig_units = get(fig, 'Units');
                set(fig, 'Units', 'pixels');
                fig_position = get(fig, 'Position');
                label = sprintf('%s [%s]', channel_name, channel_unit);
                [width, height, y_shift] = space4colorbar(fig, ...
                    channel.display_scale, label);
                fig_position(3) = width;
                fig_position(4) = height;
                shifted_position = original_position;
                shifted_position(2) = shifted_position(2) + y_shift;
                set(fig, 'Position', fig_position);
                
                % Add color bar
                add_colorbar(channel.display_scale, label);
                set(gca, 'Position', shifted_position);
                set(fig, 'Units', fig_units);
                set(gca, 'Units', ax_units);
            end
        end
    else % General scan or 1D image scan
        % Truncate pixel values to lower/upper value
        channel.pixels(channel.pixels < lower) = lower;
        channel.pixels(channel.pixels > upper) = upper;
        
        % Reverse axis if necessary
        if length(hor_axis.variable) == 1 && hor_axis.variable.extent < 0
            channel.pixels = fliplr(channel.pixels);
        end
        
        [iv_min, iv_max, ~, ~] = get_axes_limits(channel, ...
            native_options.iv1_axis, native_options.iv2_axis);
        if isnan(iv_min) || isnan(iv_max)
            me = MException('spm:spm_display:undefined_axis_limits', ...
                'Axis limits are undefined.');
            throw(me);
        end
        
        % Create line plot
        caxis([lower,upper]);
        if iv_min == iv_max &&  size(channel.pixels, 2) == 1
            if strcmp(native_options.color_lut,'minimal')
                plot(0, channel.display_scale * channel.pixels', ...
                    plot_options);
                set(gca,'Tag', 'channel_plot1d');
            else
                colormap(native_options.color_lut);
                fig_pos = channel.display_scale * channel.pixels';
                ax_pos = zeros(length(fig_pos),1);
                z = ax_pos;
                for i = 1:size(fig_pos,2)
                    coloured_line_3d(ax_pos,fig_pos(:,i),z,fig_pos(:,i));
                end
            end
        elseif iv_min ~= iv_max && size(channel.pixels, 2) > 1
            step = (iv_max - iv_min)/ (size(channel.pixels, 2) - 1);
            if strcmp(native_options.color_lut,'minimal')
                plot(iv_min:step:iv_max, ...
                    channel.display_scale * channel.pixels', plot_options);
                set(gca,'Tag', 'channel_plot1d');
            else
                colormap(native_options.color_lut);
                fig_pos = channel.display_scale * channel.pixels';
                ax_pos = iv_min:step:iv_max;
                z = zeros(length(fig_pos),1);
                hold on;
                for i = 1:size(fig_pos,2)
                    coloured_line_3d(ax_pos,fig_pos(:,i),z,fig_pos(:,i));
                end
                hold off;
            end
        else
            me = MException('spm:spm_display:plot_range_problem', ...
                ['Cannot make plot. ' , ...
                'Please check pixel number and input range']);
            throw(me);
        end
        
        % Label axes
        [hor_unit, ~, channel_unit] = get_display_units(channel, ...
            native_options.data_component);
        [hor_name, ~, channel_name] = get_axes_names(channel, ...
            native_options.data_component);
        iv_label = sprintf('%s [%s]', hor_name, hor_unit);
        dv_label = sprintf('%s [%s]',  channel_name, channel_unit);
        xlabel(iv_label);
        ylabel(dv_label);
        
        % Set limits of horizontal axis
        if iv_min < iv_max
            xlim([iv_min, iv_max]);
        else
            xlim([iv_min - eps, iv_max + eps]);
        end
        
        % Set limits of vertical axis
        ylim(channel.display_scale*[lower, upper]);
    end
catch me
    close(fig);
    rethrow(me);
end

% Attach meta-data to figure
set(fig,'UserData',channel);

% Make figure visisble
set(fig, 'visible', 'on');

% To restore imshow preferences
    function restore_iptprefs(prefs)
        iptsetpref('ImshowAxesVisible', prefs.axis_visible);
        iptsetpref('ImshowBorder', prefs.border);
        iptsetpref('ImshowInitialMagnification', prefs.magnification);
    end
end

