function to = spm_filter(from, tukey, cutoff1, varargin)
%SPM_FILTER 3rd order exp. decay filtering on Tukey-windowed data.
%    SPM_FILTER(FROM, TUKEY, CUTOFF1) carries out 3rd order exponential
%    decay low/high/band filtering after prior Tukey window application. 
%
%    SPM_FILTER(FROM, TUKEY, CUTOFF1) carries out low-pass filtering, 
%    where the input parameters have the following meaning:
%
%    FROM              Channel containing the pixel data to be
%                      filtered
%    TUKEY             Window parameter, 0 < TUKEY <= 0.5, determines the
%                      width of the area that is windowed, a useful value
%                      is 0.02. If TUKEY <= 0, no window is applied.
%    CUTOFF1           Relative cut-off frequency, can have a value ranging
%                      from 0 to 0.5. The cut-off wavelength is given by:
%                      image_size / (number_of_pixels * CUTOFF1)
%
%   SPM_FILTER(FROM, TUKEY, CUTOFF1) can also be written as
%   SPM_FILTER(FROM, TUKEY, CUTOFF1, 'Type', 'low-pass').
%
%   SPM_FILTER(FROM, TUKEY, CUTOFF1, 'Type', 'high-pass') carries out 
%   high-pass filtering.
%
%   SPM_FILTER(FROM, TUKEY, CUTOFF1, CUTOFF2, 'Type', 'band-pass') or
%   equivalently
%   SPM_FILTER(FROM, TUKEY, CUTOFF1, CUTOFF2) is used to apply band-pass 
%   filtering.
%
%   Line-by-line/global processing is controlled by the 'Mode' parameter.
%   SPM_FILTER(FROM, TUKEY, CUTOFF1, CUTOFF2, 'Mode', 'global') applies 
%   a global band-pass filter, and
%   SPM_FILTER(FROM, TUKEY, CUTOFF1, 'Type', 'low-pass', 'Mode', 'line')
%   applies a line-by-line low-pass filter. 
%   By default 'Mode' equals 'global'.

% Check input
try
    parser = inputParser;
    parser.addRequired('from', @(from)is_channel(from));
    parser.addRequired('tukey', @(tukey) isnumeric(tukey) && ...
        numel(tukey) == 1);
    parser.addRequired('cutoff1', @(cutoff1) isnumeric(cutoff1) && ...
        numel(cutoff1) == 1 && cutoff1 >= 0 && cutoff1 <= 0.5);
    parser.addOptional('cutoff2', NaN, @(cutoff2)(...
        isnumeric(cutoff2) && numel(cutoff2) == 1 && cutoff2 >= 0 && ...
        cutoff2 <= 0.5));
    parser. addParamValue('mode', 'global',@(mode)(...
        strcmp(mode,'global') || strcmp(mode,'line')));
    parser.addParamValue('type', 'low-pass',@(type)(...
        strcmp(type,'low-pass') || strcmp(type,'high-pass') || ...
        strcmp(type,'band-pass')));
    parser.parse(from, tukey, cutoff1, varargin{:});
    cutoff2 = parser.Results.cutoff2;        
    mode = parser.Results.mode;
    if ~isnan(cutoff2)
        type = 'band-pass';
    else
        type = parser.Results.type;
    end
catch error
    me = MException('spm:spm_filter:input_error', ...
        'Invalid input: %s', error.message);
    throw(me);
end

if cutoff1 > cutoff2
    me = MException('spm:spm_filter:invalid_cutoffs', ...
        'First cutoff cannot exceed second cutoff.');
    throw(me);
end

if ~isnan(cutoff2) && ~strcmp(type,'band-pass')
    me = MException('spm:spm_filter:too_many_cutoffs', ...
        'Low- and band-pass filters only require one cutoff.');
    throw(me);
end

% Copy input to output
to = from;

% Apply Tukey window
if tukey > 0
    to = tukey_window(to, tukey, mode);
end

% Filtering
cutlimit1 = fix(cutoff1 * (size(to.pixels,2)-1));

switch mode
    case 'global'
        to.pixels = fft2(to.pixels);
        switch type
            case 'low-pass'
                ff = construct_window(size(to.pixels,1), ...
                    size(to.pixels,2), 'lp',-3,cutlimit1);
                to.pixels = ff .* to.pixels;
            case 'high-pass'
                ff = construct_window(size(to.pixels,1), ...
                    size(to.pixels,2), 'hp',-3,cutlimit1);
                to.pixels = ff .* to.pixels;
            case 'band-pass'
                cutlimit2 = fix(cutoff2 * (size(to.pixels,2)-1));
                ffl = construct_window(size(to.pixels,1), ...
                    size(to.pixels,2), 'lp',-3,cutlimit2);
                ffh = construct_window(size(to.pixels,1), ...
                    size(to.pixels,2), 'hp',-3,cutlimit1);
                to.pixels =  ffh .* ffl .* to.pixels;
        end
        to.pixels = real(ifft2(to.pixels));
    case 'line'
        for r = 1 : size(to.pixels,1)
            to.pixels(r, :) = fft( to.pixels(r, :));
            switch type
                case 'low-pass'
                    ff = construct_window(1, ...
                        size(to.pixels,2), 'lp',-3,cutlimit1);
                     to.pixels(r, :) = ff .*  to.pixels(r, :);
                case 'high-pass'
                    ff = construct_window(1, ...
                        size(to.pixels,2), 'hp',-3,cutlimit1);
                     to.pixels(r, :) = ff .*  to.pixels(r, :);
                case 'band-pass'
                    cutlimit2 = fix(cutoff2 * (size(to.pixels,2)-1));
                    ffl = construct_window(1, ...
                        size(to.pixels,2), 'lp',-3,cutlimit2);
                    ffh = construct_window(1, ...
                        size(to.pixels,2), 'hp',-3,cutlimit1);
                     to.pixels(r, :) =  ffh .* ffl .*  to.pixels(r, :);
            end
            to.pixels(r, :) = real(ifft(to.pixels(r, :)));
        end
end

end

