function out = spm_hist(in, varargin)
%SPM_HIST Calculation of the histogram of a channel.
%   SPM_HIST(IN, N, LOWER, UPPER) returns an N-bin histogram of the pixel
%   values of channel IN that fall in the interval [LOWER, UPPER]. The
%   N bins are equally divided over the range of pixels that are
%   considered. SPM_HIST(IN, N) sets LOWER to the minimum value in IN and 
%   UPPER to the maximum value in IN. SPM_HIST(IN) uses the same defaults
%   for LOWER an UPPER and additionally uses 256 bins.
%
%   Beware that by default, spm_display will display the resulting
%   histogram using axis type 'relative', which implies that the start
%   value of the horizontal axis is shifted to zero. To let the axis start 
%   at the center of the left-most bin, one should therefore explicilty set 
%   the axis type to 'fine': spm_display(histogram, 'iv1_axis', 'fine').

% Check input
try
    parser = inputParser;
    parser.addRequired('in', @is_channel);
    parser.addOptional('n', 256, @(n) isnumeric(n) && ...
        numel(n) == 1 && n > 0 && floor(n) == n);
    parser.addOptional('lower', in.display_scale *min(in.pixels(:)), ...
        @(lower) isnumeric(lower) && numel(lower) == 1);
    parser.addOptional('upper', in.display_scale * max(in.pixels(:)), ...
        @(upper) isnumeric(upper) && ...
        numel(upper) == 1);
    parser.parse(in, varargin{:});
    lower = parser.Results.lower;
    upper = parser.Results.upper;
    n = parser.Results.n;
catch error
    me = MException('spm:spm_hist:input_error', ...
        'Invalid input: %s', error.message);
    throw(me);
end

if lower >  upper
    me = MException('spm:spm_hist:boundary_error', ...
        'Lower boundary should not be greater than upper boundary.');
    throw(me);
end

% Copy input channel
out = in;

% Convert lower and upper values from display units to actual units
lower = lower / in.display_scale;
upper = upper / in.display_scale;

% Reset axes and parameters
out.axis(1).name = in.name;
out.axis(1).count = n;
out.axis(1).direction = 'forward';
out.axis(1).interval = 0;


if strcmp(in.type, 'generalScanStandard')
    [~, vert_axis,~, ~] = get_axes_and_params(in);
    if ~isempty(vert_axis)
        out.axis(2) = vert_axis;
    end
else
    out.axis = out.axis(1);
end

i = length(out.parameter);
out.axis(1).variable.ref_index = i + 1;
out.parameter(i + 1).name = in.name;
out.parameter(i + 1).drift = NaN;
out.parameter(i + 1).shift = NaN;
out.parameter(i + 1).unit = in.unit;
out.parameter(i + 1).display_unit = in.display_unit;
out.parameter(i + 1).display_scale = in.display_scale;

% Set channel meta-data
out.name = get_histogram_name(in); 
out.unit =  '1';
out.display_unit = '1';
out.display_scale = 1;
out.type = 'generalScanStandard';
out.pixel_type = 'real';

% Construct histogram
step = (upper - lower) / n;
bin_centers = lower + step/2:step:upper;

if strcmp(in.type, 'generalScanStandard')
    out.pixels = zeros(size(in.pixels, 1), n);
    for i = 1: size(in.pixels, 1)
        histdata = in.pixels(i,in.pixels(i,:) >= lower & ...
            in.pixels(i,:) <= upper);
        %out.pixels(i,:) = histc(histdata,lower:step:upper);
        %[out.pixels(i,:), bin_centers] = hist(histdata,n);
        if n > 1
            [out.pixels(i,:)] = hist(histdata,bin_centers);
        else
            [out.pixels(i,:)] = length(histdata);
        end
    end
else
    histdata = in.pixels(in.pixels >= lower & ...
        in.pixels <= upper);
    %out.pixels = histc(histdata,lower:step:upper);
    %[out.pixels,bin_centers]  = hist(histdata,n);
    if n > 1
        out.pixels  = hist(histdata,bin_centers);
    else
        out.pixels  = length(histdata);
    end
end

out.axis(1).variable.extent = bin_centers(end) - bin_centers(1);
out.parameter(i + 1).start_value.coarse = 0;
out.parameter(i + 1).start_value.fine = bin_centers(1);
out.parameter(i + 1).start_value.real = 'NaN';

end

