function channel_array = spm_read(filename, varargin)
%SPM_READ Read an SXM/Nanoscan XML/Nanonis file into a channel array.
%   SPM_READ(FILENAME, TYPE) returns an array of channels containing the
%   header and pixel data of the file FILENAME, which is of type TYPE
%   ('sxm' or 'xml').
%
%   A "permanent" history file is created if it does not already exist.
%	A line reference pointing is recorded to the current line of that
%   file, so that relevant command lines from a processing session
%   can be saved later when writing a channel to file.
%
%   If TYPE is omitted it is inferred from the extension of FILENAME. If
%   inference is not possible, an error message is issued and the function
%   is aborted.
%
%   Note that by default al NaN in a data matrix are replaced by the
%   average of all other data elements (or by 0 in the degenerate case that 
%   no such data elements exist). To switch this feature off, use the 
%   'replaceNaN'-parameter: spm_read(FILENAME, TYPE, 'replaceNaN', false)  
%   or spm_read(FILENAME, 'replaceNaN', false).

% Check input
try
    parser = inputParser;
    filename = get_full_path(filename);
    parser.addRequired('filename', @(filename)exist(filename, 'file'));
    parser.addOptional('type', 'automatic', ...
        @(type)ismember(type, {'sxm', 'xml'}));
    parser.addParamValue('replaceNaN', true, ...
        @(replaceNaN)ismember(replaceNaN, [0,1]));
    parser.parse(filename, varargin{:});
catch error
    me = MException('spm:spm_read:input_error', ...
        'Invalid input: %s', error.message);
    throw(me);
end

type = parser.Results.type;
replaceNotaNumber = parser.Results.replaceNaN;

if strcmp(type, 'automatic')
    [~, ~, ext] = fileparts(filename);
    if ~isempty(ext)
        type = lower(ext(2:end));
    else
        type = '';
    end
end

if strcmp(type, 'sxm') ~= 1 && strcmp(type, 'xml') ~= 1
    if isempty(type)
        me = MException('spm:spm_read:type_unknown', ...
            'Unknown file type.');
    else
        me = MException('spm:spm_read:type_unknown', ...
            'File type %s is unknown.', type);
    end
    throw(me);
end


% Read file
full_path = mfilename('fullpath');
root_path = fileparts(full_path);
if strcmp(type, 'xml') == 1 % XML
    parameters.context = filename;
    if exist(fullfile(root_path, 'xml',  'nanoscan2empa.xsl'), 'file')
        stylesheet = fullfile(root_path, 'xml',  'nanoscan2empa.xsl');
        compiled = false;
    elseif exist(fullfile(root_path, 'xml',  'nanoscan2empa.sxx'), 'file')
        stylesheet = fullfile(root_path, 'xml',  'nanoscan2empa.sxx');
        compiled = true;
    else
        me = MException('spm:spm_read:missing_stylesheet', ...
            'Cannot find NanoScan->EMPA conversion stylesheet.');
        throw(me);
    end
    empa_schema = ...
        [root_path, filesep, 'xml', filesep, 'empa.xsd']; %#ok<NASGU>
    [~,is_empa_file] = evalc('xml_validate(filename, empa_schema)');
    if is_empa_file
        domnode = xmlread(filename);
    else
        try
            domnode = xslt_flex(filename, stylesheet, 'saxon', ..., 
                compiled, parameters);
        catch e
            me = MException('spm:spm_read:invalid_xml', ...
                'Cannot convert to Empa XML.');
            disp(e.message);
            throw(me);
        end
    end
    channel_array = dom2channels(domnode, filename);
else % (Empa) SXM
    try
        [~, channel_array] = evalc('sxm2channels(filename)');
    catch
        try
            [~, channel_array] = evalc('nanonis2channels(filename)');
        catch me_nanonis_sxm
            me = MException('spm:spm_read:invalid_sxm', ...
                'File is not a proper EMPA or Nanonis SXM file');
            throw(me);
            disp(me_empa_sxm.message);
            disp(me_nanonis_sxm.message);
        end
    end
end

% Apply unit conversion
channel_array = convert2si(channel_array);

% Apply NaN replacement
if replaceNotaNumber
    [channel_array, replaced] = replace_nan(channel_array);
    for i = 1: length(replaced)
        warning('spm:spm_read:replaced_nans', ...
            'Replaced NaNs in channel %d', replaced(i));
    end
end

end
