function to = square_magnetization(x_size, y_size, x_period, y_period, ...
     nx, ny, unit)
% SQUARE_MAGNETIZATION Creation of a square magnetization pattern.
%   SQUARE_MAGNETIZATION(X_SIZE,Y_SIZE, X_PERIOD, Y_PERIOD, NX, NY, UNIT) 
%   returns an up/down or +-1 magnetization pattern that consists of 
%   overlayed square waves in the x and y-direction. The input parameters 
%   have the following meaning:
%
%   X_SIZE      width of the image
%   Y_SIZE      height of the image
%   X_PERIOD    period of the wave in the x-direction
%   Y_PERIOD    period of the wave in the y-direction
%   NX          number of pixels in x-direction
%   NY          number of pixels in y-direction
%   UNIT        axis unit (sizes and periods are displayed in this unit)
%
%   Setting a period to a value greater than the size in the corresponding
%   direction results in a flat unit function in this direction. This means
%   that no wave is generated.
%
%   The internal axis units of the output channel are set to 'm' where
%   possible. If no automatic conversion can be done, the internal unit is
%   set to the display unit. In this case, a warning is issued.

% Check input
try
    parser = inputParser;
    parser.addRequired('x_size', @(x_size)(numel(x_size) == 1 && ...
        isnumeric(x_size)));
    parser.addRequired('y_size', @(y_size)(numel(y_size) == 1 && ...
        isnumeric(y_size)));
     parser.addRequired('x_period', @(x_period)(numel(x_period) == 1 && ...
        isnumeric(x_period)));
    parser.addRequired('y_period', @(y_period)(numel(y_period) == 1 && ...
        isnumeric(y_period)));
    parser.addRequired('nx', @(nx)(numel(nx) == 1 && isnumeric(nx) && ...
        floor(nx) == nx));
    parser.addRequired('ny', @(nx)(numel(ny) == 1 && isnumeric(ny) && ...
        floor(ny) == ny));
    parser.addRequired('unit', @ischar);
    parser.parse(x_size, y_size, x_period, y_period, nx, ny, unit);
catch error
    me = MException('spm:square_magnetization:input_error', ...
        'Invalid input: %s', error.message);
    throw(me);
end

% First set output to the empty image
to = empty_image(x_size, y_size, nx, ny, unit);

% Determine pixel periods
x_pix_period = 2 * round(nx * x_period / (2 * x_size));
y_pix_period = 2 * round(ny * y_period / (2* y_size));

% Issues warning in case period got changed to match an integer and even 
% number
if x_pix_period * x_size ~= x_period * nx 
    warning('spm:spm_display:x_period_change', ...
        ['Changing x_period to %g to match an', ...
        ' integer and even number of pixels'], ...
        x_pix_period * x_size / nx);
end

if y_pix_period * y_size ~= y_period * ny 
    warning('spm:spm_display:y_period_change', ...
        ['Changing y_period to %g to match an', ...
        ' integer and even number of pixels'], ...
        y_pix_period * y_size / ny);
end

% Define tiles
white = ones(y_pix_period / 2, x_pix_period / 2);
black = -ones(y_pix_period / 2, x_pix_period / 2);

% Combine tiles to 2x2 pattern
pattern = [white, black; black , white];

% Determine number of patterns
nof_x_patterns = ceil(nx / x_pix_period);
nof_y_patterns = ceil(ny / y_pix_period);

% Repeat pattern
to.pixels = repmat(pattern, nof_y_patterns, nof_x_patterns);

% Cut out right size
to.pixels = to.pixels(1 : ny, 1 : nx);

end
