function to = tukey_window(from, alpha, varargin)
% TUKEY_WINDOW  Applies a Tukey window to a channel.
%   TUKEY_WINDOW(FROM, ALPHA, MODE) with MODE = 'global' computes the
%   result of applying a Tukey window to the spatial channel FROM with
%   window parameter ALPHA. Such a window can be used to reduce artifacts 
%   when estimating the power spectrum. (See e.g. "Digital  signal 
%   processing: a practical approach", E.C. Ifeachor and B.W. Jervis,
%   chapter 10.) The Tukey window tapers toward the average value of the
%   image. To apply a 1-D Tukey window in a line-by-line fashion, set MODE
%   = 'line'. By default,  MODE = 'global'.

% Check input
try
    parser = inputParser;
    parser.addRequired('in', @(in)is_channel(in));
    parser.addRequired('alpha', @(alpha) isnumeric(alpha) && ... 
        numel(alpha) == 1 && alpha > 0);
    parser.addOptional('mode', 'global', @(mode)(...
        strcmp(mode,'global') || strcmp(mode,'line')));
    parser.parse(from, alpha, varargin{:});
    mode = parser.Results.mode;
catch error
    me = MException('spm:tukey_window:input_error', ...
        'Invalid input: %s', error.message);
    throw(me);
end

% Prepare the process
work_image = from.pixels;
s = size(work_image);
nx = s(2);
ny = s(1);

% Generate the window
x = abs(fix((0 : nx - 1) - nx / 2.0 + 0.1));
% Possible division by zero?
x = min(circshift(x, [0,fix(nx / 2.0)]) * pi /(nx * alpha), pi);
x = 0.5 - 0.5 * cos(x);

y = abs(fix((0 : ny - 1) - ny / 2.0 + 0.1));
y = min(circshift(y, [0,fix(ny / 2.0)]) * pi /(ny * alpha), pi);
y = 0.5 - 0.5 * cos(y);

% Do the actual windowing
switch mode
    case 'global'
        av_value = mean(work_image(:));
        work_image = av_value + (work_image - av_value) .* (y' * x);
    case 'line'
        for r = 1: size(work_image,1)
             av_value = mean(work_image(r,:));
             work_image(r, :) = av_value + ...
                 (work_image(r, :) - av_value) .* x;
        end
end

% Set range and write to output channel
to = from;
to.pixels = work_image;

end